### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2013 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces

# import local interfaces
from ztfy.skin.interfaces import ICustomBackOfficeInfoTarget

# import Zope3 packages
from zope.component import queryAdapter
from zope.traversing import namespace

# import local packages
from ztfy.utils.traversing import getParent


class CustomBackOfficeInfoNamespace(namespace.view):
    """++back++ namespace traverser"""

    def traverse(self, name, ignored):
        target = getParent(self.context, ICustomBackOfficeInfoTarget)
        if target is not None:
            return queryAdapter(target, target.back_interface)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from zope.contentprovider.interfaces import IContentProvider

# import local interfaces
from ztfy.skin.interfaces import IFormViewletsManager, IFormPrefixViewletsManager, IWidgetsPrefixViewletsManager, \
                                 IWidgetsSuffixViewletsManager, IFormSuffixViewletsManager
from ztfy.skin.layer import IZTFYBrowserLayer

# import Zope3 packages
from z3c.template.template import getViewTemplate
from zope.component import adapts
from zope.interface import implements, Interface
from zope.viewlet.viewlet import ViewletBase as Viewlet
from zope.viewlet.manager import ViewletManagerBase as ViewletManager, WeightOrderedViewletManager

# import local packages


class ViewletManagerBase(ViewletManager):

    template = getViewTemplate()


class WeightViewletManagerBase(WeightOrderedViewletManager):

    template = getViewTemplate()


class ViewletBase(Viewlet):

    render = getViewTemplate()


class ContentProviderBase(object):

    adapts(Interface, IZTFYBrowserLayer, Interface)
    implements(IContentProvider)

    def __init__(self, context, request, view):
        self.context = context
        self.request = request
        self.__parent__ = view

    def update(self):
        pass

    render = getViewTemplate()


class FormViewletManager(WeightOrderedViewletManager):

    implements(IFormViewletsManager)


class FormPrefixViewletManager(FormViewletManager):

    implements(IFormPrefixViewletsManager)


class WidgetsPrefixViewletManager(FormViewletManager):

    implements(IWidgetsPrefixViewletsManager)


class WidgetsSuffixViewletManager(FormViewletManager):

    implements(IWidgetsSuffixViewletsManager)


class FormSuffixViewletManager(FormViewletManager):

    implements(IFormSuffixViewletsManager)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from z3c.template.interfaces import ILayoutTemplate, IPageTemplate
from zope.publisher.interfaces.browser import IBrowserSkinType

# import local interfaces

# import Zope3 packages
from z3c.template.template import getPageTemplate, getLayoutTemplate
from zope.component import getMultiAdapter, getUtility, queryUtility
from zope.interface import implements
from zope.publisher.browser import BrowserPage
from zope.publisher.skinnable import applySkin

# import local packages
from ztfy.skin.interfaces import IBaseForm


class BaseTemplateBasedPage(BrowserPage):

    template = getPageTemplate()

    def update(self):
        pass

    def render(self):
        if self.template is None:
            template = getMultiAdapter((self, self.request), IPageTemplate)
            return template(self)
        return self.template()

    def __call__(self):
        self.update()
        return self.render()


class TemplateBasedPage(BaseTemplateBasedPage):

    layout = getLayoutTemplate()

    def __call__(self):
        self.update()
        if self.layout is None:
            layout = getMultiAdapter((self, self.request), ILayoutTemplate)
            return layout(self)
        return self.layout()


class BaseBackView(object):
    """Base back-office view, automatically selecting the good skin"""

    implements(IBaseForm)

    def update(self):
        skin = queryUtility(IBrowserSkinType, 'ZMI')
        if (skin is None) or ((skin is not None) and not skin.providedBy(self.request)):
            applySkin(self.request, getUtility(IBrowserSkinType, 'ZTFY.BO'))



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages

# import Zope3 interfaces
from jquery.layer import IJQueryJavaScriptBrowserLayer
from z3c.form.interfaces import IFormLayer
from z3c.jsonrpc.layer import IJSONRPCLayer
from z3c.layer.pagelet import IPageletBrowserLayer

# import local interfaces

# import Zope3 packages

# import local packages


class IBaseZTFYLayer(IFormLayer, IPageletBrowserLayer):
    """Base ZTFY browser layer"""


class IZTFYBrowserLayer(IBaseZTFYLayer, IJQueryJavaScriptBrowserLayer, IJSONRPCLayer):
    """ZTFY JavaScript layer"""


class IZTFYBackLayer(IZTFYBrowserLayer):
    """ZTFY back-office layer"""


class IZTFYFrontLayer(IZTFYBrowserLayer):
    """ZTFY front-office layer"""



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2013 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from zope.dublincore.interfaces import IZopeDublinCore

# import local interfaces

# import Zope3 packages
from zope.traversing.api import getName

# import local packages
from ztfy.skin.viewlet import ViewletBase


class TitleViewlet(ViewletBase):
    """Title viewlet"""

    @property
    def title(self):
        try:
            title = self.__parent__.title
        except AttributeError:
            title = getattr(self.context, 'title', None)
            if title is None:
                dc = IZopeDublinCore(self.context, None)
                if dc is not None:
                    title = dc.title
        if not title:
            title = '[ %s ]' % getName(self.context)
        return title
